@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("2.7.") or
-    raise "ruby lib version (2.7.3) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (2.7.4) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.7/powerpc-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "2"
   CONFIG["MINOR"] = "7"
-  CONFIG["TEENY"] = "3"
-  CONFIG["PATCHLEVEL"] = "183"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.7.3/bin/install -c'
+  CONFIG["TEENY"] = "4"
+  CONFIG["PATCHLEVEL"] = "191"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.7.4/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)27"
@@ -44,7 +44,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=27' '--with-soname=ruby27' '--with-ruby-version=minor' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=27' '--with-soname=ruby27' '--with-ruby-version=minor' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -149,9 +149,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.7.4/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.7.4/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.7.4/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -165,41 +165,41 @@ module RbConfig
   CONFIG["AR"] = "ar"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 10.0.1 \nTarget: powerpc-unknown-openbsd6.9\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 11.1.0\nTarget: powerpc-unknown-openbsd7.0\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["MJIT_CC"] = "/usr/bin/cc"
   CONFIG["CSRCFLAG"] = ""
   CONFIG["COUTFLAG"] = "-o "
   CONFIG["OUTFLAG"] = "-o "
   CONFIG["CPPOUTFILE"] = "-o conftest.i"
-  CONFIG["GNU_LD"] = "yes"
+  CONFIG["GNU_LD"] = "no"
   CONFIG["LD"] = "ld"
   CONFIG["GCC"] = "yes"
+  CONFIG["CXXFLAGS"] = "-O2 -pipe"
+  CONFIG["CXX"] = "c++"
   CONFIG["EGREP"] = "/usr/bin/egrep"
   CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["CPP"] = "$(CC) -E"
-  CONFIG["CXXFLAGS"] = "-O2 -pipe"
-  CONFIG["CXX"] = "c++"
   CONFIG["OBJEXT"] = "o"
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E"
   CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd6.9"
+  CONFIG["target_os"] = "openbsd7.0"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "powerpc"
-  CONFIG["target"] = "powerpc-unknown-openbsd6.9"
-  CONFIG["host_os"] = "openbsd6.9"
+  CONFIG["target"] = "powerpc-unknown-openbsd7.0"
+  CONFIG["host_os"] = "openbsd7.0"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "powerpc"
-  CONFIG["host"] = "powerpc-unknown-openbsd6.9"
+  CONFIG["host"] = "powerpc-unknown-openbsd7.0"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd6.9"
+  CONFIG["build_os"] = "openbsd7.0"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "powerpc"
-  CONFIG["build"] = "powerpc-unknown-openbsd6.9"
+  CONFIG["build"] = "powerpc-unknown-openbsd7.0"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
   CONFIG["HAVE_GIT"] = "no"
