@@ -5,25 +5,18 @@ require 'rubygems/version_option'
 require 'rubygems/gemcutter_utilities'
 
 class Gem::Commands::YankCommand < Gem::Command
+
   include Gem::LocalRemoteOptions
   include Gem::VersionOption
   include Gem::GemcutterUtilities
 
   def description # :nodoc:
     <<-EOF
-The yank command removes a gem you pushed to a server from the server's
-index.
-
-Note that if you push a gem to rubygems.org the yank command does not
-prevent other people from downloading the gem via the download link.
+The yank command permanently removes a gem you pushed to a server.
 
 Once you have pushed a gem several downloads will happen automatically
-via the webhooks.  If you accidentally pushed passwords or other sensitive
+via the webhooks. If you accidentally pushed passwords or other sensitive
 data you will need to change them immediately and yank your gem.
-
-If you are yanking a gem due to intellectual property reasons contact
-http://help.rubygems.org for permanent removal.  Be sure to mention this
-as the reason for the removal request.
     EOF
   end
 
@@ -40,9 +33,11 @@ as the reason for the removal request.
 
     add_version_option("remove")
     add_platform_option("remove")
+    add_otp_option
 
     add_option('--host HOST',
-               'Yank from another gemcutter-compatible host') do |value, options|
+               'Yank from another gemcutter-compatible host',
+               '  (e.g. https://rubygems.org)') do |value, options|
       options[:host] = value
     end
 
@@ -58,7 +53,7 @@ as the reason for the removal request.
     version   = get_version_from_requirements(options[:version])
     platform  = get_platform_from_requirements(options)
 
-    if version then
+    if version
       yank_gem(version, platform)
     else
       say "A version argument is required: #{usage}"
@@ -68,7 +63,10 @@ as the reason for the removal request.
 
   def yank_gem(version, platform)
     say "Yanking gem from #{self.host}..."
-    yank_api_request(:delete, version, platform, "api/v1/gems/yank")
+    args = [:delete, version, platform, "api/v1/gems/yank"]
+    response = yank_api_request(*args)
+
+    say response.body
   end
 
   private
@@ -77,6 +75,7 @@ as the reason for the removal request.
     name = get_one_gem_name
     response = rubygems_api_request(method, api, host) do |request|
       request.add_field("Authorization", api_key)
+      request.add_field("OTP", options[:otp]) if options[:otp]
 
       data = {
         'gem_name' => name,
@@ -86,7 +85,7 @@ as the reason for the removal request.
 
       request.set_form_data data
     end
-    say response.body
+    response
   end
 
   def get_version_from_requirements(requirements)
@@ -100,4 +99,3 @@ as the reason for the removal request.
   end
 
 end
-
