@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("2.7.") or
-    raise "ruby lib version (2.7.3) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (2.7.4) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.7/sparc64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "2"
   CONFIG["MINOR"] = "7"
-  CONFIG["TEENY"] = "3"
-  CONFIG["PATCHLEVEL"] = "183"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.7.3/bin/install -c'
+  CONFIG["TEENY"] = "4"
+  CONFIG["PATCHLEVEL"] = "191"
+  CONFIG["INSTALL"] = '/build/tmp/pobj//ruby-2.7.4/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)27"
@@ -149,9 +149,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.7.3/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/build/tmp/pobj//ruby-2.7.4/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/build/tmp/pobj//ruby-2.7.4/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/build/tmp/pobj//ruby-2.7.4/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -175,11 +175,11 @@ module RbConfig
   CONFIG["GNU_LD"] = "yes"
   CONFIG["LD"] = "ld"
   CONFIG["GCC"] = "yes"
-  CONFIG["EGREP"] = "/usr/bin/egrep"
-  CONFIG["GREP"] = "/usr/bin/grep"
-  CONFIG["CPP"] = "$(CC) -E"
   CONFIG["CXXFLAGS"] = "-O2 -pipe"
   CONFIG["CXX"] = "c++"
+  CONFIG["EGREP"] = "/usr/bin/egrep"
+  CONFIG["GREP"] = "/usr/bin/grep"
+  CONFIG["CPP"] = "cc -E"
   CONFIG["OBJEXT"] = "o"
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E"
@@ -202,7 +202,7 @@ module RbConfig
   CONFIG["build"] = "sparc64-unknown-openbsd6.9"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["HAVE_GIT"] = "no"
+  CONFIG["HAVE_GIT"] = "yes"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
